Package: tabnet
Title: Fit 'TabNet' Models for Classification and Regression
Version: 0.9.1
Authors@R: c(
    person("Daniel", "Falbel", , "daniel@rstudio.com", role = "aut"),
    person(, "RStudio", role = "cph"),
    person("Christophe", "Regouby", , "christophe.regouby@free.fr", role = c("cre", "ctb")),
    person("Egill", "Fridgeirsson", role = "ctb"),
    person("Philipp", "Haarmeyer", role = "ctb"),
    person("Sven", "Verweij", role = "ctb",
           comment = c(ORCID = "0000-0002-5573-3952"))
  )
Description: Implements the 'TabNet' model by Sercan O. Arik et al. (2019)
    <doi:10.48550/arXiv.1908.07442> with 'Coherent Hierarchical
    Multi-label Classification Networks' by Giunchiglia et al.
    <doi:10.48550/arXiv.2010.10151> and provides a consistent interface
    for fitting and creating predictions.  It's also fully compatible with
    the 'tidymodels' ecosystem.
License: MIT + file LICENSE
URL: https://mlverse.github.io/tabnet/,
        https://github.com/mlverse/tabnet
BugReports: https://github.com/mlverse/tabnet/issues
Depends: R (>= 3.6)
Imports: coro, data.tree, dials, dplyr, generics, ggplot2, hardhat (>=
        1.3.0), magrittr, methods, parsnip, progress, purrr, rlang,
        stats, stringr, tibble, tidyr, torch (>= 0.4.0), tune, utils,
        vctrs, withr, zeallot
Suggests: cli, knitr, modeldata, patchwork, quarto, recipes, rmarkdown,
        rsample, spelling, testthat (>= 3.0.0), tidymodels, tidyverse,
        vip, visdat, workflows, xgboost, yardstick
VignetteBuilder: knitr
Additional_repositories: https://bgreenwell.r-universe.dev
Config/roxygen2/version: 8.0.0
Config/testthat/edition: 3
Config/testthat/parallel: false
Config/testthat/start-first: interface, explain, params
Encoding: UTF-8
Language: en-US
NeedsCompilation: no
Packaged: 2026-07-22 06:47:08 UTC; creg
Author: Daniel Falbel [aut],
  RStudio [cph],
  Christophe Regouby [cre, ctb],
  Egill Fridgeirsson [ctb],
  Philipp Haarmeyer [ctb],
  Sven Verweij [ctb] (ORCID: <https://orcid.org/0000-0002-5573-3952>)
Maintainer: Christophe Regouby <christophe.regouby@free.fr>
Repository: CRAN
Date/Publication: 2026-07-22 07:10:08 UTC
Built: R 4.5.2; ; 2026-07-22 11:49:53 UTC; unix
