# Generated by roxygen2: do not edit by hand

S3method(augment,tabnet_fit)
S3method(min_grid,tabnet)
S3method(multi_predict,"_tabnet_fit")
S3method(nn_prune_head,tabnet_fit)
S3method(nn_prune_head,tabnet_pretrain)
S3method(predict,tabnet_fit)
S3method(print,tabnet_fit)
S3method(print,tabnet_pretrain)
S3method(tabnet_explain,default)
S3method(tabnet_explain,model_fit)
S3method(tabnet_explain,tabnet_fit)
S3method(tabnet_explain,tabnet_pretrain)
S3method(tabnet_fit,Node)
S3method(tabnet_fit,data.frame)
S3method(tabnet_fit,default)
S3method(tabnet_fit,formula)
S3method(tabnet_fit,recipe)
S3method(tabnet_pretrain,Node)
S3method(tabnet_pretrain,data.frame)
S3method(tabnet_pretrain,default)
S3method(tabnet_pretrain,formula)
S3method(tabnet_pretrain,recipe)
S3method(update,tabnet)
export("%>%")
export(attention_width)
export(augment)
export(build_ancestor_matrix_from_outcomes)
export(cat_emb_dim)
export(check_compliant_node)
export(checkpoint_epochs)
export(decision_width)
export(drop_last)
export(encoder_activation)
export(entmax)
export(entmax15)
export(feature_reusage)
export(lr_scheduler)
export(mask_type)
export(mlp_activation)
export(mlp_hidden_multiplier)
export(momentum)
export(nn_aum_loss)
export(nn_mc_loss)
export(nnf_mc_loss)
export(nnf_multilabel_one_hot)
export(node_to_df)
export(num_independent)
export(num_independent_decoder)
export(num_shared)
export(num_shared_decoder)
export(num_steps)
export(optimizer)
export(penalty)
export(sparsemax)
export(sparsemax15)
export(tabnet)
export(tabnet_config)
export(tabnet_explain)
export(tabnet_fit)
export(tabnet_nn)
export(tabnet_pretrain)
export(verbose)
export(virtual_batch_size)
importFrom(dplyr,filter)
importFrom(dplyr,last_col)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_all)
importFrom(dplyr,mutate_if)
importFrom(dplyr,select)
importFrom(dplyr,starts_with)
importFrom(dplyr,where)
importFrom(generics,augment)
importFrom(magrittr,"%>%")
importFrom(parsnip,multi_predict)
importFrom(rlang,.data)
importFrom(stats,predict)
importFrom(stats,setNames)
importFrom(stats,update)
importFrom(tidyr,replace_na)
importFrom(torch,as_array)
importFrom(torch,nn_module)
importFrom(torch,nn_prune_head)
importFrom(torch,nnf_binary_cross_entropy_with_logits)
importFrom(torch,torch_argsort)
importFrom(torch,torch_cat)
importFrom(torch,torch_long)
importFrom(torch,torch_matmul)
importFrom(torch,torch_mean)
importFrom(torch,torch_minimum)
importFrom(torch,torch_mul)
importFrom(torch,torch_std)
importFrom(torch,torch_sum)
importFrom(torch,torch_tensor)
importFrom(tune,min_grid)
importFrom(zeallot,"%<-%")
