---
name: Bug report
about: Create a report to help us improve
title: ''
labels: bug
assignees: eduardocarvp, Hartorn, j-abi, Optimox

---

<!-- Please don't delete this template or we'll close your issue -->
**Describe the bug**
<!-- A clear and concise description of what the bug is. -->

**What is the current behavior?**

**If the current behavior is a bug, please provide the steps to reproduce.**
<!--
1. Go to '...'
2. Click on '....'
3. Scroll down to '....'
4. See error
-->

**Expected behavior**
<!-- "It should work" is not a good explanation -->
<!-- Explain how exactly you expecting it to behave -->

**Screenshots**
<!-- If applicable, add screenshots to help explain your problem. -->

**Other relevant information:**
poetry version:  
python version: 
Operating System: 
Additional tools:

**Additional context**
<!-- Add any other context about the problem here. -->
