---
name: Feature request
about: Suggest an idea for this project
title: ''
labels: enhancement
assignees: eduardocarvp, Hartorn, j-abi, Optimox

---

<!-- Please don't delete this template or we'll close your issue -->

## Feature request


**What is the expected behavior?**


**What is motivation or use case for adding/changing the behavior?**


**How should this be implemented in your opinion?**


**Are you willing to work on this yourself?**
yes
