% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tabnet_package.R
\docType{package}
\name{tabnet-package}
\alias{tabnet-package}
\title{tabnet: Fit 'TabNet' Models for Classification and Regression}
\description{
Implements the 'TabNet' model by Sercan O. Arik et al. (2019) \doi{10.48550/arXiv.1908.07442} with 'Coherent Hierarchical Multi-label Classification Networks' by Giunchiglia et al. \doi{10.48550/arXiv.2010.10151} and provides a consistent interface for fitting and creating predictions. It's also fully compatible with the 'tidymodels' ecosystem.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://mlverse.github.io/tabnet/}
  \item \url{https://github.com/mlverse/tabnet}
  \item Report bugs at \url{https://github.com/mlverse/tabnet/issues}
}

}
\author{
\strong{Maintainer}: Christophe Regouby \email{christophe.regouby@free.fr} [contributor]

Authors:
\itemize{
  \item Daniel Falbel \email{daniel@rstudio.com}
}

Other contributors:
\itemize{
  \item  RStudio [copyright holder]
  \item Egill Fridgeirsson [contributor]
  \item Philipp Haarmeyer [contributor]
  \item Sven Verweij (\href{https://orcid.org/0000-0002-5573-3952}{ORCID}) [contributor]
}

}
\keyword{internal}
